<?php
namespace Teknisa\CMP\Controllers;

use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Exception;
use Teknisa\CMP\Factories\Cmp99000GeracaoAfPrevisaoFactory;
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\CMP\Services\Cmp99000GeracaoAfPrevisaoService;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;

class Cmp99000GeracaoAfPrevisaoController extends
    \Teknisa\GER\Controllers\TeknisaCrud
{
    protected $factory;
    protected $service;
    protected $crudService;
    protected $parameterBag;
    protected $environment;

    public function __construct(
        $crudService,
        $parameterBag,
        $environment,
        $factory,
        $service
    ) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment = $environment;
        $this->factory = $factory;
        $this->service = $service;
    }

    public function getGeracaoAfPrevisao(Row $request, Response $response)
    {
        
        try {
            $params = $request->getParameters();
            $params = json_decode($params['filters']);
            
      
            if ($params->NRAPURPREC == null || $params->NRAPURPREC == '') {
                $params->NRAPURPREC = $this->service->generateCode(
                    "APURPRECO" . $params->CDFILCENTCMP,
                    $params->NRORG
                );

                $rowsAffect = $this->factory->gerarProcedure($params);
                if($rowsAffect['P_ROWSAFFECT'] == "N"){
                   $response->setError(new Error("Não há dados para Geração de AF.", 500));
                   
                   return;
                }
            }
            
            $data = $this->factory->getItmaPurPrecData($params);
            
            $response->addDataSet(new DataSet('data', $data));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getSessionInfos(Row $request, Response $response)
    {
        $sessionInfos = [
            'CDFILIAL' => $this->environment->getCdFilial(),
            'CDOPERADOR' => $this->environment->getCdOperador()
        ];

        $response->addDataSet(new DataSet('data',  $sessionInfos));
    }
    
    public function getMask(Row $request, Response $response)
    {
      $data = $this->factory->getMask();
        
        $response->addDataSet(new DataSet('data',  $data));
    }
    
    public function updateProducts(Row $request, Response $response) {
        try {
            $rows = $request->getParameters();

            foreach ($rows as $row) {
                if(isset($row['QTSOLIPROD'])){
                    $row['originalRow']['QTSOLIPROD'] = $row['QTSOLIPROD'];
                }
                
                if(isset($row['VRCUSTO'])){
                    $row['originalRow']['VRCUSTO'] = $row['VRCUSTO'];
                }
                $this->factory->updateProducts($row['originalRow']);
                $this->factory->updateProducts1($row['originalRow']);
            }
            
            $response->addDataSet(new DataSet('data', array( "success"  => true)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function verifyBeforeGenerate(Row $request, Response $response) {
       
            $params = $request->getParameters();
            $rows=array();
            
            if(isset($params['edit'])){
               foreach($params['rows'] as $row) {
                $param=[];
                $param['geraAf'] =$params['geraAf']?? null;
                $param['rows']=$row;
                $param['filters']=$params['filters'];
                
                array_push($rows, $this->factory->verifyBeforeGenerateProcedureEdited($param));
                
                }
            } else { 
                $rows= $this->factory->verifyBeforeGenerateProcedure($params['filters']);
                
            }
            
              
            if (count($rows) > 0) {
                $formattedRows = array();
                
                foreach($rows as $row) {
                    if (isset($row[0])) {
                    $dserro = $this->factory->errorDescription(explode("_", $row[0]['DSERRO']));
                    $row[0]['DSERRO'] = $dserro;
                    
                    array_push($formattedRows, $row[0]);
                    } else if(isset($row['DSERRO'])){
                        $dserro = $this->factory->errorDescription(explode("_", $row['DSERRO']));
                        $row['DSERRO'] = $dserro;
                    
                        array_push($formattedRows, $row);
                    }
                }
                $response->addDataSet(new DataSet('data', $formattedRows));
                
                return;
               
            }
            
            $response->addDataSet(new DataSet('data', []));
       
    }
    public function callDeleteProcedure(Row $request, Response $response) {
       
            $params = $request->getParameters();
            
            $rows = $this->factory->callDeleteProcedure($params['filter']) ?? [];
             if($rows['P_ROWSAFFECT'] == "N"){
               $response->setError(new Error("Não há dados para Geração de AF.", 500));
             }
            $response->addDataSet(new DataSet('data', $rows));
       
    }
    
      public function uploadFile(Row $request, Response $response) {
       
            $params = $request->getParameters();
            
            $table = $this->xlsRead($params);
        

            
            $response->addDataSet(new DataSet('data', $table));

       
    }
    
    private function xlsRead($row)
    {
        try {
            $worksheet = $this->readFile($row);
            $r = $this->saveTableValues($worksheet);

            $v = $this->addFieldsFromFilter($r,$row['row']['FILTER']);
         
            $results = [];
            
                foreach ($v as $element) {
                  
             $rowsAffect = $this->factory->gerarProcedureForm($element);
           
              $rowsAffect['DSERRO'] = $this->factory->errorDescription(explode("_", $rowsAffect['DSERRO']));
              $rowsAffect['CDFORNECED_OLD']='';
              $rowsAffect['VRCUSTO']=floatval($rowsAffect['VRCUSTO']);
             $results[] = $rowsAffect;
                }
            // $file = $row['FILE'];
            // $fileName = $file['name'];

            // list($name, $extension) = explode(".", $fileName);
            // if (strtolower($extension) === 'xls') {
            //     throw new \Exception("INVALID_FORMAT_XLS");
            // }

            // $row = [
            //     "STATUS" => 'PENDENTE',
            //     "DTIMPORTACAO" => date("d/m/Y"),
            //     "RESPONSAVEL" => $this->environment->getCdoperador(),
            //     "DSARQUIVO" => $fileName,
            //     "CDOPERLOGADO" => $this->environment->getCdoperador(),
            //     "CDFILIAL" => $this->environment->getCdFilial()
            // ];

            // $row = $this->insertHeader($worksheet, $row);
            // $table = $this->insertBody($worksheet, $row);

            return $results;
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    function saveTableValues($worksheet) {
        
    $data = [];
    $worksheetDimensions = $worksheet->getHighestRowAndColumn();
    $maxRowIndex = $worksheetDimensions['row'];
    $maxColumnIndex = Coordinate::columnIndexFromString($worksheetDimensions['column']);
    $fromSistema = false;
    // Detect units dynamically from row 1
    $unidades = [];
    for ($col = 6; $col <= $maxColumnIndex; $col += 2) {
        if($worksheet->getCellByColumnAndRow($col, 2)->getValue()!='EST'){
            $col++;
            $fromSistema =true;
        }
        $unidade = $worksheet->getCellByColumnAndRow($col, 1)->getValue();
        if ($unidade) {
            $unidades[$col] = $unidade;
        }
    }
    
    // Iterate over rows starting from row start here)
    for ($row = 3; $row <= $maxRowIndex; $row++) {
        $codigo = $worksheet->getCell('A' . $row)->getValue();
        $fornecedor = strval($worksheet->getCell('C' . $row)->getValue());
        $fornecedor= explode(' ',$fornecedor)[0];
        if(strlen($fornecedor)>12 && strlen($fornecedor)<14){
         $fornecedor=   str_pad($fornecedor, 14, '0', STR_PAD_LEFT);
        }
        if($fromSistema){
            $vrCusto = $worksheet->getCell('F' . $row)->getValue(); 
        } else {
            $vrCusto = $worksheet->getCell('D' . $row)->getValue(); 
        }
        //  $vrCusto = str_replace('.', '', $vrCusto);
    
        // $vrCusto = str_replace('.', ',', $vrCusto);
        // var_dump($vrCusto);
        foreach ($unidades as $col => $unidade) {
            $estValue = $worksheet->getCellByColumnAndRow($col, $row)->getValue();
            $cmpValue = $worksheet->getCellByColumnAndRow($col + 1, $row)->getValue();
        //   var_dump($col);
            
                $data[] = [
                    'QTSOLIPROD' => floatval($cmpValue),
                    'CDARVPROD' =>  preg_replace('/[^a-zA-Z0-9]/', '', $codigo),
                    'NRINSJURFORN' => $fornecedor,
                    'DSFILIAL' => $unidade,
                    'VRCUSTO' => floatval($vrCusto),
                    'posicao' => Coordinate::stringFromColumnIndex($col + 1) . $row
                ];
            
        }
       
    }
    // var_dump($data); die;
    return $data;
      
    
}

function addFieldsFromFilter($data, $filter) {
    $fields = [
        "CDCOMPRADOR" => isset($filter["CDCOMPRADOR"]) ? $filter["CDCOMPRADOR"] : null,
        "CDFILCENTCMP" => isset($filter["CDFILCENTCMP"]) ? $filter["CDFILCENTCMP"] : null,
        "CDFILIAL" => isset($filter["CDFILIAL"]) ? $filter["CDFILIAL"] : "",
        "CDFORNECED" => isset($filter["CDFORNECED"]) ? $filter["CDFORNECED"] : "",
        "CDFORNERELACAF" => isset($filter["CDFORNERELACAF"]) ? $filter["CDFORNERELACAF"] : "",
        "CDMARCA" => isset($filter["CDMARCA"]) ? $filter["CDMARCA"] : null,
        "CDPRODFIN" => isset($filter["CDPRODFIN"]) ? $filter["CDPRODFIN"] : null,
        "CDPRODINI" => isset($filter["CDPRODINI"]) ? $filter["CDPRODINI"] : null,
        "CDSUBGRUPPROD" => isset($filter["CDSUBGRUPPROD"]) ? $filter["CDSUBGRUPPROD"] : "",
        "CDTIPOCUSTO" => isset($filter["CDTIPOCUSTO"]) ? $filter["CDTIPOCUSTO"] : "",
        "DTENTREGA" => isset($filter["DTENTREGA"]) ? $filter["DTENTREGA"] : "",
        "DTFINAL" => isset($filter["DTFINAL"]) ? $filter["DTFINAL"] : "",
        "DTINICIAL" => isset($filter["DTINICIAL"]) ? $filter["DTINICIAL"] : "",
        "NRAPURPREC" =>  "",
        "NRORG" => isset($filter["NRORG"]) ? $filter["NRORG"] : null,
        "QTDIAESTOQUE" => isset($filter["QTDIAESTOQUE"]) ? $filter["QTDIAESTOQUE"] : null,
        "TIPOPREVISAO" => isset($filter["TIPOPREVISAO"]) ? $filter["TIPOPREVISAO"] : "",
        "TIPOTELA" =>  "C",
        "UM_REGISTRO" => isset($filter["UM_REGISTRO"]) ? $filter["UM_REGISTRO"] : "",
        "CDFILLAN" => isset($filter["CDFILLAN"]) ? $filter["CDFILLAN"] : "",
        "CDFILSOLI" => isset($filter["CDFILSOLI"]) ? $filter["CDFILSOLI"] : "",
        "CDPRODUTO" => isset($filter["CDPRODUTO"]) ? $filter["CDPRODUTO"] : "",
        "DTPROGENTR" => isset($filter["DTPROGENTR"]) ? $filter["DTPROGENTR"] : "",
        "NMPRODUTO" => isset($filter["NMPRODUTO"]) ? $filter["NMPRODUTO"] : "",
        "NMPRODUTOOL" => isset($filter["NMPRODUTOOL"]) ? $filter["NMPRODUTOOL"] : null,
        "NRSOLICMP" => isset($filter["NRSOLICMP"]) ? $filter["NRSOLICMP"] : "",
       
    ];
         
    foreach ($data as &$row) {
        $row = array_merge($fields, $row);
    }

    return $data;
}


    
    private function readFile($row)
    {
       
        try {
            $file = $row['row']['FILE'];
            $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);

            if (!$xlsFile) {
                throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);
            }

            return $xlsFile->getActiveSheet();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function generateAfs(Row $request, Response $response) {
        try {
            $this->factory->beginTransaction();
            
            $params = $request->getParameters();
   
            $rows = $params['rows'];
            
            if(count($rows) > 0) {
                foreach ($rows as $row) {
                    $this->factory->updateProductStats($row, 'S');
                }
                
                $nrApurPrec = $rows[0]["NRAPURPREC"];
                $cdFilcent = $rows[0]["CDFILCENTCMP"];
                
                $efetivarAfParams = [
                    "CDOPERADOR" => $params['CDOPERADOR'],
                    "DTAUTOFORN" => date('d/m/Y'),
                    "CDFILCENTCMP" => $cdFilcent,
                    "NRAPURPREC" => $nrApurPrec
                ];
    
                $procedure = $this->factory->efetivarAfProcedure($efetivarAfParams);
                
                $autFor = $this->factory->retornaAutFor($nrApurPrec, $cdFilcent);
                
                if(count($autFor) <= 0) {
                  $erroGeral['erroGeral'] = 'A';
                    
                  $response->addDataSet(new DataSet('data', $erroGeral));
                  
                  return;
                }
                
                $arrErro = array();
                $erroGeral = array('erroGeral' => 'N');
             
                foreach ($autFor as $row) {
                    $erro = $this->factory->procedureFECHA_AUTOFORNE_MOBILE($row);
                    /*$this->factory->editarStatusImportAutoForn($row);*/
                    array_push($arrErro, $erro);
                    if($erro['P_SEM_ERRO'] == 'N'){   
                        $erroGeral = array('erroGeral' => 'S');
                    }
                }
                array_push($arrErro, $erroGeral);
                
                $checkValida = $this->factory->getBlockedAF($cdFilcent);
        
                if($erroGeral['erroGeral'] == 'S'){
                    if($checkValida[0]['IDBLOQUEAAF'] == 'S'){
                      $this->factory->realizaRollback();
                      $erroGeral = array('erroGeral' => 'R');
                      
                      $response->addDataSet(new DataSet('data', $erroGeral));
                      return;
                    }
                }
                
               // $this->factory->editarStatusImportApurPreco($rows[0]);
               
               foreach($rows as $key => $row) {
                $this->factory->updateProductStats1($row, 'S');
               } 
               
               $this->factory->realizaCommit();
        
               $response->addDataSet(new DataSet('data', $erroGeral));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->factory->realizaRollback();
        }
    }
    
    public function getDetails(Row $request, Response $response) {
        try{
            $params = $request->getParameters();
            $params = json_decode($params['filters']);
                
            $details = $this->factory->getDetails($params);
            
            $statusTypes = [
                'NF' => $this->factory::$nfStatus,
                'PE' => $this->factory::$peStatus,
                'AF' => $this->factory::$afStatus,
                'RE' => $this->factory::$reStatus,
            ];
            
            $status = $statusTypes[$params->TIPO];
            
            $formattedDetails = [];
            
            if (count($details) > 0) {
              foreach($details as $detail) {
                  $detail['STATUS'] = $status[$detail['STATUS']];
                  $formattedDetails[] = $detail;
              }
            }
         
            $response->addDataSet(new DataSet('data', $formattedDetails)); 
            
        } catch(\Exception $e) {
           $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getTaxesDetails(Row $request, Response $response) {
        try{
            $params = $request->getParameters();
            $params = json_decode($params['filters']);
                
            $taxes = $this->factory->getTaxesDetails($params);
            
            $response->addDataSet(new DataSet('data', $taxes)); 
            
        } catch(\Exception $e) {
           $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function generateProcedure(Row $request, Response $response) {
        try {
            $params = (object) $request->getParameters();
    
            if ($params->NRAPURPREC == null || $params->NRAPURPREC == '') {
                $params->NRAPURPREC = $this->service->generateCode(
                    "APURPRECO" . $params->CDFILCENTCMP,
                    $params->NRORG
                );
            }
           
            $rowsAffect = $this->factory->gerarProcedure($params);

            if($rowsAffect['P_ROWSAFFECT'] == "N"){
               $response->setError(new Error("Não há dados para Geração de AF.", 500));
               
               return;
            }

            $response->addDataSet(new DataSet('data', ['success']));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveEditedRows(Row $request, Response $response) {
        try {
            $rows = $request->getParameters();
        
            foreach ($rows['rows'] as $row) {
                //$row['VRCUSTO'] = isset($row['VRCUSTOBKP']) ? $row['VRCUSTOBKP'] : $row['VRCUSTO'];
                $str = trim($row['VRCUSTO']);
                $str = str_replace(',', '.', $str);
                
                $row['VRCUSTO']   = (float)$str;
                // var_dump($row);
                 $produto= $this->factory->updateProducts($row);
                 $produto= $this->factory->updateProducts1($row);
            }
            
            $response->addDataSet(new DataSet('data', array( "success"  => true)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
}
