<?php
namespace Teknisa\CMP\Factories;

use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Cmp99000GeracaoAfPrevisaoFactory
{
    protected $entityManager;
    protected $connection;
      
    public function __construct($entityManager, $environment) {
      $this->entityManager  = $entityManager;
      $this->connection     = $entityManager->getConnection();
      $this->environment    = $environment;
    }

    public function realizaCommit(){
       $this->connection->commit();
    }

    public function realizaRollback(){
        $this->connection->rollBack();
    }
    
    public function beginTransaction(){
        $this->entityManager->beginTransaction();
    }

    private function executeQuery($sql, $params){
        
        $this->entityManager->beginTransaction();

        $std = $this->connection->prepare($sql);
     
        $std->execute($params);

        return $std;
    }
    
    public static $nfStatus = [
        'P' => 'Pendente',
        'A' => 'Aprovado',
        'E' => 'Emitido',
        'C' => 'Cancelado',
        'V' => 'Previsto',
        'G' => 'Gerencial',
        'R' => 'Rateio',
        'L' => 'Liberado'
    ];
    
    public static $peStatus = [
        1 => 'Pendente',
        2 => 'Confirmado',
        3 => 'Aprovado',
        4 => 'Liquidado',
        5 => 'Cancelado',
        6 => 'Liquidação Forçada',
        7 => 'Parcialmente Atendido'
    ];
    
    public static $afStatus = [
        1 => 'Baixa Parcial',
        2 => 'Baixa Total',
        3 => 'Baixa Manual',
        null => 'Em Aberto'
    ];
    
    public static $reStatus = [
        1 => 'Pendente',
        2 => 'Confirmado',
        3 => 'Aprovado',
        4 => 'Liquidado',
        5 => 'Cancelado',
        6 => 'Liquidação Forçada',
        7 => 'Parcialmente Atendido'
    ];
    
    public function errorDescription($errors) {
        $errorsDescription = [
            'ERROREG'=> 'Unidade sem Região cadastrada.',
            'ERROINATIV'=> 'Unidade com Contrato Rescindido.',
            'ERROSUP'=> 'Central de Compras sem Parametrização de Compras e Suprimento.',
            'ERROCMP'=> 'Unidade sem Parametrização de Compras e Suprimentos.',
            'ERROMARCAPDR'=> 'Não existe marca padrão cadastrada.',
            'ERRODTENR'=> 'Data de entrega menor que a data de processamento da unidade.',
            'ERROREFOREG'=> 'Fornecedor selecionado não atende a região da Unidade.',
            'ERROREFORFIL'=> 'Fornecedor selecionado não atende a Unidade.',
            'QUALIFORN'=> 'Fornecedor selecionado não qualificado.',
            'FORNECINATIV'=> 'Fornecedor selecionado é inativo ou alternativo.',
            'PRODFORNCOMP'=> 'Fornecedor selecionado não relacionado ao produto.',
            'MARCPRODCOMP'=> 'Marca selecionado não relacionado ao produto.',
            'QFOPROFFORN'=> 'Fornecedor selecionado não qualificado para o produto.',
            'QFOMARPROD'=> 'Marca selecionado não qualificada para o produto.',
            'RELACPRODOL' => 'Produto selecionado sem relacionamento do OL.',
            'ITEMAUTF' => 'Produto com AF em Aberto para os últimos 30 dias.',
            'NRAPURPREC' => 'Produto com AF gerada para apuração atual.',
            'CDFORNITMAP' => 'Já existe registro para o fornecedor selecionado.',
            
            'ERROCDARVPROD'=> 'Produto não cadastrado.',
            'ERROCDARVPRODV'=> 'Produto de ser informado.',
            'ERROCDFILSOLI'=> 'Unidade não cadastrada.',
            'ERROCDFILSOLIV'=> 'Unidade deve ser informada.',
            'ERRONRINSJURFORN'=> 'Fornececedor não cadastrado.',
            'ERRONRINSJURFORNV'=> 'Fornececedor de ser informado.',
            'ERROQTSOLIPRODZ' => 'Quantidade a comprar deve ser maior que zero.',
            'ERROQTSOLIPROD' => 'Quantidade a comprar inválida.',
            'ERROVRCUSTOZ' => 'Custo do item a comprar deve ser maior que zero.',
            'ERROVRCUSTO' => 'Custo do item a comprar inválido.',
            'ERROIDPRODATIV' => 'Produto inativo para a Unidade.',
            'ERROCDREGIAO' => 'Unidade sem região cadastrada.',
            'ERROIDCNTRRESC' => 'Unidade com contrato rescindido.',
            'ERROCDARVPRODI' => 'Produto Inativo.'
        ];
        
         $errorDescription = '';
  
        foreach($errors as $error){
           if($error != ''){
              $errorDescription = $errorDescription . $errorsDescription[$error] . PHP_EOL;    
           }
        }
    
        return $errorDescription;
    }

   public function getItmaPurPrecData($row){
      try {
           $sql = "SELECT ID, IDCHECK, NRAPURPREC, NVL(CDFORNECED, '') AS CDFORNECED, CDFILSOLI, NMFILIAL, CDFORNECED_OLD, DSFILIALC, DSFILIAL, CDFILLAN, NRSOLICMP, CDPRODUTO, NMPRODUTO, CDARVPROD, SGUNIDADE, DSPRODUTO, CDMARCA, NMMARCA, DSMARCA, NRINSJURFORN, NMFANTFORN, CDEXPRODFORN, DSTIPO, NMPRODUTOOL, CDARVPRODOL, CDFILCENTCMP, SGESTADO, NMMUNICIPIO, IDITMSELPREVCMP, NMCOMPRZSFIL, DTPROGENTR, NRORG, QTSOLIPROD, VRCUSTO, VRPRECUNIAP, QTDIAESTOQUE, QTCONSUMODIA, QTESTOQUE, QTDURACAEST, QTCONSUMODIASEST, QTITEMPED, QTPROGENTAF, QTENTRADAS, QTSAIDAS, VRCUSTOTOTAL, REPLACE(LTRIM(TO_CHAR(QTTOTAL, '999,999,999,999,999,999,999,999')), ',', '.') QTTOTAL, CONTADOR,CDPRODINTE, QTTOTALAF, QTTOTALCMP, VRTOTALFILCMP, REPLACE(LTRIM(TO_CHAR(VRTOTALCMP, '999,999,999,999,999,999,999,999')), ',', '.') VRTOTALCMP, VRTOTALFILEST, REPLACE(LTRIM(TO_CHAR(VRTOTALEST, '999,999,999,999,999,999,999,999')), ',', '.') VRTOTALEST, VRCUSTOEST FROM TABLE ( QUERY_ITEM_CMP99000(:CDFILCENTCMP, :NRAPURPREC, :CDFORNECED, :CDMARCA, :CDFILIAL, :CDPRODINI, :CDPRODFIN, :TIPOTELA, :CDTIPOCUSTO, :P_UM_REGISTRO) ) ORDER BY CASE WHEN :TIPOTELA = 'G' THEN CDFILSOLI || ' - ' || NMFILIAL ELSE CASE WHEN LENGTH(NMCOMPRZSFIL || SGFILIAL) <> 0 THEN LPAD(NVL(NMCOMPRZSFIL, '0'), 2, '0') || ' - ' || RPAD(SUBSTR(NVL(SGFILIAL, ' '), 1, 4), 4, ' ') || ' - ' || NMFILIAL ELSE CDFILSOLI || ' - ' || NMFILIAL END END, CDPRODINTE, NMPRODUTO";

                
                $params = [
                //"CDFORNERELACAF" => $row["CDFORNERELACAF"] ?? null,
               "CDFORNECED" => $row->CDFORNECED ?? null,
               "CDFILIAL" => $row->CDFILIAL,
               // "CDCOMPRADOR" => $row->CDCOMPRADOR,
               // "TIPOPREVISAO" => $row->TIPOPREVISAO,
               // "CDSUBGRUPPROD" => $row->CDSUBGRUPPROD[0] ?? null,
                "CDPRODINI" => $row->CDPRODINI,
                "CDPRODFIN" => $row->CDPRODFIN,
                "CDMARCA" => $row->CDMARCA,
               // "DTNICIAL" => $row->DTNICIAL,
               // "QTDIAESTOQUE" => $row->QTDIAESTOQUE,
               // "DTENTREGA" => $row->DTENTREGA ,
                "CDTIPOCUSTO" => $row->CDTIPOCUSTO,
                "NRAPURPREC" => $row->NRAPURPREC ?? null,
                "CDFILCENTCMP" => $row->CDFILCENTCMP ?? null,
                "TIPOTELA" => $row->TIPOTELA,
                "P_UM_REGISTRO" => $row->UM_REGISTRO
            ];
            
            $std = $this->executeQuery($sql, $params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function callDeleteProcedure($params) {
        try {
           
       
            $rowsAffect = "";
            $procedure  = new StoredProcedure($this->connection, 'EXCLUI_AF_POR_PREVISAO_COMPRA');

            $procedure->addParam(new Param('P_CDFILCENTCMP', Param::PARAM_INPUT, isset($params['CDFILCENTCMP']) ? $params['CDFILCENTCMP']: null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRAPURPREC', Param::PARAM_INPUT, isset($params['NRAPURPREC']) ? $params['NRAPURPREC'] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, isset($params['CDFORNECED']) ? $params['CDFORNECED']: null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, isset($params['CDFILIAL']) ? $params['CDFILIAL'] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDCOMPRADOR', Param::PARAM_INPUT, isset($params['CDCOMPRADOR']) ? $params['CDCOMPRADOR']: null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODINI', Param::PARAM_INPUT, isset($params['CDPRODINI']) ? $params['CDPRODINI'] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODFIN', Param::PARAM_INPUT, isset($params['CDPRODFIN']) ? $params['CDPRODFIN'] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, isset($params['CDMARCA']) ? $params['CDMARCA']: null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTENTREGA', Param::PARAM_INPUT, isset($params['DTENTREGA']) ? $params['DTENTREGA'] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT', Param::PARAM_OUTPUT, $rowsAffect, Param::PARAM_TYPE_STR));
            $rowsAffect = $procedure->execute();
       
            return $rowsAffect;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    public function gerarProcedure($params) {
        try {
            
            $stDataIni =  $params->DTINICIAL;
            $stDataFin  = $params->DTFINAL;
            
            if($params->DTENTREGA==null){
                $stDataIni = null;
                $stDataFin = null;
            }
            else if (strlen($params->DTFINAL) < 9) {
               $stDataIni = '01/'.$stDataIni;
               $stDataFin =  '01/'.$stDataFin;
            }
            $rowsAffect = "";
            $procedure  = new StoredProcedure($this->connection, 'GERA_AF_POR_PREVISAO_COMPRA');

            $procedure->addParam(new Param('P_CDFILCENTCMP', Param::PARAM_INPUT, isset($params->CDFILCENTCMP) ? $params->CDFILCENTCMP: null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRAPURPREC', Param::PARAM_INPUT, isset($params->NRAPURPREC) ? $params->NRAPURPREC : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNERELACAF', Param::PARAM_INPUT, isset($params->CDFORNERELACAF) ? $params->CDFORNERELACAF :  null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, isset($params->CDFORNECED) ? $params->CDFORNECED : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, isset($params->CDFILIAL) ? $params->CDFILIAL : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDCOMPRADOR', Param::PARAM_INPUT, isset($params->CDCOMPRADOR) ? $params->CDCOMPRADOR: null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TIPOPREVISAO', Param::PARAM_INPUT, isset($params->TIPOPREVISAO) ? $params->TIPOPREVISAO: null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDSUBGRUPPROD', Param::PARAM_INPUT, isset($params->CDSUBGRUPPROD) ? $params->CDSUBGRUPPROD : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODINI', Param::PARAM_INPUT, isset($params->CDPRODINI) ? $params->CDPRODINI : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODFIN', Param::PARAM_INPUT, isset($params->CDPRODFIN) ? $params->CDPRODFIN : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, isset($params->CDMARCA) ? $params->CDMARCA: null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, isset($stDataIni) ? $stDataIni : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, isset($stDataFin) ? $stDataFin : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTENTREGA', Param::PARAM_INPUT, isset($params->DTENTREGA) ? $params->DTENTREGA : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDTIPOCUSTO', Param::PARAM_INPUT, isset($params->CDTIPOCUSTO) ? $params->CDTIPOCUSTO : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_QTDIAESTOQUE', Param::PARAM_INPUT, isset($params->QTDIAESTOQUE) ? $params->QTDIAESTOQUE : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT', Param::PARAM_OUTPUT, $rowsAffect, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_QTSOLIPROD', Param::PARAM_INPUT, isset($params->QTSOLIPROD) ? $params->QTSOLIPROD : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_VRCUSTO', Param::PARAM_INPUT, isset($params->VRCUSTO) ? $params->VRCUSTO : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $this->environment->getCdFilial(), Param::PARAM_TYPE_STR));
    
            $rowsAffect = $procedure->execute();
       
            return $rowsAffect;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function gerarProcedureForm($row){
        try{
            $sql = "SELECT DSFILIAL, CDFILSOLI, CDPRODUTO, NMPRODUTO, CDARVPROD, DSERRO, CDFORNECED, IDTPIJURFORN, NRINSJURFORN, NMRAZSOCFORN, NMFANTFORN,:P_VRCUSTO as VRCUSTO,:P_QTSOLIPROD as QTSOLIPROD, POSICAO
  FROM TABLE (QUERY_VDITEM1_CMP99000(:P_DSFILIAL, :P_NRINSJURFORN, :P_CDARVPROD, :P_POSICAO, :P_QTSOLIPROD, :P_VRCUSTO))";

            // $stDataIni =  $row["DTINICIAL"];
            // $stDataFin  = $row["DTFINAL"];
            
            // if($row["DTENTREGA"]==null){
            //     $stDataIni = null;
            //     $stDataFin = null;
            // }
            // else if (strlen($stDataIni) < 9) {
            //   $stDataIni = '01/'.$stDataIni;
            //   $stDataFin =  '01/'.$stDataFin;
            // }
            
            $params = [
               "P_DSFILIAL" => $row["DSFILIAL"] ?? null,
               "P_NRINSJURFORN" => $row["NRINSJURFORN"] ?? null,
               "P_CDARVPROD" => $row['CDARVPROD'],
               "P_POSICAO" => $row["posicao"] ?? null,
               "P_QTSOLIPROD" => $row["QTSOLIPROD"] ?? null,
               "P_VRCUSTO" => $row["VRCUSTO"] ?? null
            ];
                 
                 
            $std = $this->executeQuery($sql, $params);
     
           return $std->fetch();
        }
        catch(\Exception $e){
           throw $e;
        }
    }
    
    public function updateProducts($row) {
        try {
            $sql="UPDATE ITMAPURPREC
                SET
                    QTACOMP = :P_QTSOLIPROD,
                    QTSOLACOMP = :P_QTSOLIPROD,
                    VRPRECUNIAP  = :P_VRCUSTO,
                    VRPRECECOAP  = :P_VRCUSTO,
                    VRPRECONPIAP  = :P_VRCUSTO
                WHERE
                        CDCENTITMAP = :P_CDFILCENTCMP
                    AND NRSEQITMAP = :P_NRAPURPREC
                    AND CDFORNITMAP = :P_CDFORNECED
                    AND DTENTRAPUT = :P_DTENTRAPUT
                    AND CDFILIAP = :P_CDFILSOLI
                    AND CDPRODITMAP = :P_CDPRODUTO
                    AND CDMARCITMAP = :P_CDMARCA
                    AND NVL(IDITMSELPREVCMP, 'N') = 'N'
                    AND NVL(IDITMSELAPUR, 'N') = 'N'";
                    
            $params = [
                "P_QTSOLIPROD"     => $row['QTSOLIPROD'],
                "P_VRCUSTO"        => $row['VRCUSTO'],
                "P_CDFILCENTCMP"   => $row['CDFILCENTCMP'],
                "P_NRAPURPREC"     => $row['NRAPURPREC'],
                "P_CDFORNECED"     => $row['CDFORNECED_OLD'],
                "P_DTENTRAPUT"     => $row['DTPROGENTR'],
                "P_CDFILSOLI"      => $row['CDFILSOLI'],
                "P_CDPRODUTO"      => $row['CDPRODUTO'],
                "P_CDMARCA"        => $row['CDMARCA'],
            ];
            
            
            $query = $this->executeQuery($sql, $params);
            
            $this->realizaCommit();
            return  $query;
        } catch (\Exception $e) {
            $this->realizaRollback();  
            
            throw $e;
        }
    }
    
   
    public function updateProducts1($row) {
        if($row['CDFORNECED']!= $row['CDFORNECED_OLD']){
            try {
            
                $sql = "UPDATE ITMAPURPREC
                    SET
                        CDFORNITMAP   = :P_CDFORNECED
                    WHERE
                            CDCENTITMAP = :P_CDFILCENTCMP
                        AND NRSEQITMAP = :P_NRAPURPREC
                        AND CDFORNITMAP = :P_CDFORNECED_OLD
                        AND CDFORNITMAP <> :P_CDFORNECED
                        AND DTENTRAPUT = :P_DTENTRAPUT
                        AND CDFILIAP = :P_CDFILSOLI
                        AND CDPRODITMAP = :P_CDPRODUTO
                        AND CDMARCITMAP = :P_CDMARCA
                        AND NVL(IDITMSELPREVCMP, 'N') = 'N'
                        AND NVL(IDITMSELAPUR, 'N') = 'N'"; 
            
                $params = [
                    "P_CDFILCENTCMP"   => $row['CDFILCENTCMP'],
                    "P_NRAPURPREC"     => $row['NRAPURPREC'],
                    "P_CDFORNECED"     => $row['CDFORNECED'],
                    "P_DTENTRAPUT"     => $row['DTPROGENTR'],
                    "P_CDFILSOLI"      => $row['CDFILSOLI'],
                    "P_CDPRODUTO"      => $row['CDPRODUTO'],
                    "P_CDMARCA"        => $row['CDMARCA'],
                    "P_CDFORNECED_OLD" => $row['CDFORNECED_OLD']
                ];
                
                $query = $this->executeQuery($sql, $params);
                
                $this->realizaCommit();
                return  $query;
            } catch (\Exception $e) {
                $this->realizaRollback();  
                
                throw $e;
            }
        }   
    }
    
     public function updateProductStats1($row, $status) {
        try{
            $sql = "UPDATE ITMAPURPREC
                        SET
                            IDITMSELPREVCMP = :P_IDITMSELAPUR
                        WHERE
                                CDCENTITMAP = :P_CDFILCENTCMP
                            AND NRSEQITMAP = :P_NRAPURPREC
                            AND CDFORNITMAP = :P_CDFORNECED
                            AND DTENTRAPUT = :P_DTENTRAPUT
                            AND CDFILIAP = :P_CDFILSOLI
                            AND CDPRODITMAP = :P_CDPRODUTO
                            AND CDMARCITMAP = :P_CDMARCA"; 
                    
            $params = [
                "P_CDFILCENTCMP"   => $row['CDFILCENTCMP'],
                "P_NRAPURPREC"     => $row['NRAPURPREC'],
                "P_CDFORNECED"     => $row['CDFORNECED'],
                "P_DTENTRAPUT"     => $row['DTPROGENTR'],
                "P_CDFILSOLI"      => $row['CDFILSOLI'],
                "P_CDPRODUTO"      => $row['CDPRODUTO'],
                "P_CDMARCA"        => $row['CDMARCA'],
                "P_IDITMSELAPUR"   => $status
            ];
        
            $std = $this->connection->prepare($sql);
        
            $std->execute($params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updateProductStats($row, $status) {
       try{
        $sql = "UPDATE ITMAPURPREC
            SET
                IDITMSELAPUR = :P_IDITMSELAPUR
            WHERE
                    CDCENTITMAP = :P_CDFILCENTCMP
                AND NRSEQITMAP = :P_NRAPURPREC
                AND CDFORNITMAP = :P_CDFORNECED
                AND DTENTRAPUT = :P_DTENTRAPUT
                AND CDFILIAP = :P_CDFILSOLI
                AND CDPRODITMAP = :P_CDPRODUTO
                AND CDMARCITMAP = :P_CDMARCA";
                
        $params = [
            "P_CDFILCENTCMP"   => $row['CDFILCENTCMP'],
            "P_NRAPURPREC"     => $row['NRAPURPREC'],
            "P_CDFORNECED"     => $row['CDFORNECED'],
            "P_DTENTRAPUT"     => $row['DTPROGENTR'],
            "P_CDFILSOLI"      => $row['CDFILSOLI'],
            "P_CDPRODUTO"      => $row['CDPRODUTO'],
            "P_CDMARCA"        => $row['CDMARCA'],
            "P_IDITMSELAPUR"   => $status
        ];
          
          $std = $this->connection->prepare($sql);
        
          $std->execute($params);

        } catch (\Exception $e) {
            throw $e;
        }
   }
   
    
    public function efetivarAfProcedure($efetivarAfParams) {
       try{
        $procedure  = new StoredProcedure($this->connection, 'EFETIVAR_AF'); 

        $procedure->addParam(new Param('P_CDOPERADOR',  Param::PARAM_INPUT,   $efetivarAfParams['CDOPERADOR'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTAUTOFORN',  Param::PARAM_INPUT,   $efetivarAfParams['DTAUTOFORN'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCENTCMP',  Param::PARAM_INPUT,   $efetivarAfParams['CDFILCENTCMP'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQAPUR',  Param::PARAM_INPUT,   $efetivarAfParams['NRAPURPREC'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',  Param::PARAM_INPUT,   $this->environment->getCdFilial(),  Param::PARAM_TYPE_STR));
        $procedure->execute();
    
        }
        catch(\Exception $e){
            throw $e;
        }
        
    }    
    
      public function retornaAutFor($nrApurPrec, $cdFilcent) {
        try{
        $sql = "SELECT CDFILDEST, CDFILLANC, NRAUTOFORN FROM AUTOFORNE, FILIAL WHERE AUTOFORNE.CDFILDEST = FILIAL.CDFILIAL AND NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = :P_CDFILCENTCMP AND AUTOFORNE.NRSEQAPURAF = :P_NRAPURPREC";
        $params = [
                'P_NRAPURPREC' => $nrApurPrec,
                'P_CDFILCENTCMP' => $cdFilcent
            ];
     
        $std = $this->connection->prepare($sql);

        $std->execute($params);
            
        return $std->fetchAll();
        }
        catch(\Exception $e){
            throw $e;
        }
        
    }
    
        public function procedureFECHA_AUTOFORNE_MOBILE($row) {
         try{
           $P_SEM_ERRO = "";
           $procedure  = new StoredProcedure($this->connection, 'FECHA_AUTOFORNE_MOBILE'); 
           $procedure->addParam(new Param('P_CDFILDEST',  Param::PARAM_INPUT,   $row['CDFILDEST'],  Param::PARAM_TYPE_STR));
           $procedure->addParam(new Param('P_CDFILLANC',  Param::PARAM_INPUT,   $row['CDFILLANC'],  Param::PARAM_TYPE_STR));
           $procedure->addParam(new Param('P_NRAUTOFORN',  Param::PARAM_INPUT,   $row['NRAUTOFORN'],  Param::PARAM_TYPE_STR));
           $procedure->addParam(new Param('P_SEM_ERRO',      Param::PARAM_OUTPUT,  $P_SEM_ERRO,           Param::PARAM_TYPE_STR));
           $P_SEM_ERRO = $procedure->execute();

           return  $P_SEM_ERRO; 
        }
        catch(\Exception $e){
            throw $e;
        }
    }  
    
       public function editarStatusImportAutoForn($row) {
        try{
           $sql = "UPDATE AUTOFORNE
                 SET IDBAIXAAF = NULL
                WHERE CDFILDEST = :CDFILDEST
                  AND CDFILLANC = :CDFILLANC
                  AND NRAUTOFORN = :NRAUTOFORN";
                
        $params = [
            "CDFILDEST"   => $row['CDFILDEST'],
            "CDFILLANC"     => $row['CDFILLANC'],
            "NRAUTOFORN"     => $row['NRAUTOFORN']
       ];
         $std = $this->connection->prepare($sql);

         $std->execute($params);
        
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
       
    public function getBlockedAF($params) {
        try{
        $sql = "SELECT PARASUPRI.IDBLOQUEAAF FROM PARASUPRI WHERE PARASUPRI.CDFILIAL = :P_CDFILCENTCMP";
                 
        $params =
            [
                'P_CDFILCENTCMP' => $params
            ];
    
        $std = $this->connection->prepare($sql);

        $std->execute($params);
            
        
        return $std->fetchAll();
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function editarStatusImportApurPreco($params) {
           try{
       $sql = "UPDATE APURPRECO  SET IDSITAPUPREC = '1'  WHERE CDFILCENTCMP =  :CDFILCENTCMP    AND NRAPURPREC = :NRAPURPREC";
                
        $params = [
            'NRAPURPREC' => $params['NRAPURPREC'],
            'CDFILCENTCMP' => $params['CDFILCENTCMP']
      ];
       
        $std = $this->connection->prepare($sql);

        $std->execute($params);
        
        }
        catch(\Exception $e){
            throw $e;
        }
    }
    
    public function verifyBeforeGenerateProcedure($row){
        try{
            $sql = "SELECT :P_CDFILCENTCMP CDFILCENTCMP, :P_NRAPURPREC NRAPURPREC, DSFILIAL, CDPRODUTO, NMPRODUTO, CDARVPROD, DSERRO, NRORG 
            FROM TABLE(QUERY_VDITEM_CMP99000(:P_CDFILCENTCMP , :P_CDFORNECED , :P_CDFILIAL , :P_CDCOMPRADOR , :P_TIPOPREVISAO , :P_CDSUBGRUPPROD , :P_CDPRODINI , :P_CDPRODFIN , :P_CDMARCA , :P_DTINICIAL , :P_DTFINAL , :P_DTENTREGA , :P_TIPOTELA , :P_CDOPERADOR, :P_NRAPURPREC, :P_GERA_AF))";

            $stDataIni =  $row["DTINICIAL"];
            $stDataFin  = $row["DTFINAL"];
            
            if($row["DTENTREGA"]==null){
                $stDataIni = null;
                $stDataFin = null;
            }
            else if (strlen($stDataIni) < 9) {
               $stDataIni = '01/'.$stDataIni;
               $stDataFin =  '01/'.$stDataFin;
            }
            
            $params = [
               "P_CDFORNECED" => $row["CDFORNECED"] ?? null,
               "P_CDFILIAL" => $row["CDFILIAL"] ?? null,
               "P_CDFILCENTCMP" => $row['CDFILCENTCMP'],
               "P_CDSUBGRUPPROD" => $row["CDSUBGRUPPROD"] ?? null,
               "P_CDCOMPRADOR" => $row["CDCOMPRADOR"] ?? null,
               "P_TIPOPREVISAO" => $row["TIPOPREVISAO"] ?? null,
               "P_CDPRODINI" => $row["CDPRODINI"] ?? null,
               "P_CDPRODFIN" => $row["CDPRODFIN"] ?? null,
               "P_CDMARCA" => $row["CDMARCA"] ?? null,
               "P_DTINICIAL" => $stDataIni ?? null,
               "P_DTFINAL" => $stDataFin ?? null,
               "P_DTENTREGA" => $row["DTENTREGA"] ?? null,
               "P_TIPOTELA" => $row["TIPOTELA"] ?? null,
               "P_CDOPERADOR" => $this->environment->getCdOperador(),
               "P_NRAPURPREC" => $row["NRAPURPREC"] ?? null,
                "P_GERA_AF" =>  $row["geraAf"] ?? "N"
            ];
                 
            $std = $this->executeQuery($sql, $params);
     
           return $std->fetchAll();
        }
        catch(\Exception $e){
           throw $e;
        }
    }
    
     public function verifyBeforeGenerateProcedureEdited($row){
        try{
            $sql = "SELECT :P_CDFILCENTCMP CDFILCENTCMP, :P_NRAPURPREC NRAPURPREC, DSFILIAL, CDPRODUTO, NMPRODUTO, CDARVPROD, DSERRO, NRORG 
            FROM TABLE(QUERY_VDITEM_CMP99000(:P_CDFILCENTCMP , :P_CDFORNECED , :P_CDFILIAL , :P_CDCOMPRADOR , :P_TIPOPREVISAO , :P_CDSUBGRUPPROD , :P_CDPRODINI , :P_CDPRODFIN , :P_CDMARCA , :P_DTINICIAL , :P_DTFINAL , :P_DTENTREGA , :P_TIPOTELA , :P_CDOPERADOR, :P_NRAPURPREC, :P_GERA_AF))";
            
            
            
            
            $stDataIni =  $row['filters']["DTINICIAL"];
            $stDataFin  = $row['filters']["DTFINAL"];
            
            if($row['filters']["DTENTREGA"]==null){
                $stDataIni = null;
                $stDataFin = null;
            }
            else if (strlen($stDataIni) < 9) {
               $stDataIni = '01/'.$stDataIni;
               $stDataFin =  '01/'.$stDataFin;
            }
            
            $params = [
               "P_CDFORNECED" => $row['rows']["CDFORNECED"] ?? null,
               "P_CDFILIAL" => $row['rows']["CDFILSOLI"] ?? null,
               "P_CDFILCENTCMP" => $row['rows']['CDFILCENTCMP'],
               "P_CDSUBGRUPPROD" => $row['filters']["CDSUBGRUPPROD"] ?? null,
               "P_CDCOMPRADOR" => $row['filters']["CDCOMPRADOR"] ?? null,
               "P_TIPOPREVISAO" => $row['filters']["TIPOPREVISAO"] ?? null,
               "P_CDPRODINI" => $row['rows']["CDARVPROD"] ?? null,
               "P_CDPRODFIN" => $row['rows']["CDARVPROD"] ?? null,
               "P_CDMARCA" => $row['rows']["CDMARCA"] ?? null,
               "P_DTINICIAL" => $stDataIni ?? null,
               "P_DTFINAL" => $stDataFin ?? null,
               "P_DTENTREGA" => $row['filters']["DTENTREGA"] ?? null,
               "P_TIPOTELA" => $row['filters']["TIPOTELA"] ?? null,
               "P_CDOPERADOR" => $this->environment->getCdOperador(),
               "P_NRAPURPREC" => $row['rows']["NRAPURPREC"] ?? null,
               "P_GERA_AF" => $row["geraAf"] ?? "N"
            ];
                 
            $std = $this->executeQuery($sql, $params);
     
           return $std->fetchAll();
        }
        catch(\Exception $e){
           throw $e;
        }
    }
    
    public function getMask() {
        try {
            $sql = "SELECT CDPICTPROD FROM PARAMGERAL";
            $params = [
            ];
            $std = $this->executeQuery($sql,$params);

           return $std->fetchAll();
        } catch(\Exception $e) {
           throw $e;
        }
    }
    
    public function getDetails($row) {
        try {
            $sql = "SELECT NRSEQUENCIAL, INSCRICAODEST, DSDESTINATARIO, CDARVPROD, NMPRODUTO, CDMARCA, DSMARCA ,STATUS, DTENTREGA, QUANTIDADE, PRECO, CHAVE, CHAVE||CDARVPROD||CDMARCA||DTENTREGA ID   
            FROM TABLE( QUERY_DETALHE_CMP99000(:P_CDFILIAL , :P_CDARVPROD , :P_DTINICIAL , :P_DTFINAL , :P_TIPO, :P_CDMARCA ))
            ORDER BY DTENTREGA DESC, NRSEQUENCIAL, CDARVPROD, DSMARCA";

            $params = [
                "P_CDFILIAL" => $row->CDFILIAL ?? null,
                "P_CDMARCA" => $row->CDMARCA ?? null,
                "P_CDARVPROD" => $row->CDARVPROD ?? null,
                "P_DTINICIAL" => $row->DTINICIAL ?? null,
                "P_DTFINAL" => $row->DTFINAL ?? null,
                "P_TIPO" => $row->TIPO ?? null
            ];
             
            $std = $this->executeQuery($sql, $params);

           return $std->fetchAll();
        } catch(\Exception $e) {
           throw $e;
        }
    }
    
    public function getTaxesDetails($row){
        try{
            $sql = "SELECT CDIMPOSTO, NMIMPOSTO, VRIMPOSTO, VRALIQUOTA FROM TABLE(QUERY_IMPOSTO_CMP99000(:P_NRSEQUENCIAL , :P_CDFILIAL , :P_CDARVPROD, :P_CDMARCA, :P_DTENTREGA, :P_TIPO )) ORDER BY CDIMPOSTO";

            $params = [
                "P_NRSEQUENCIAL" => $row->NRSEQUENCIAL ?? null,
                "P_CDARVPROD" => $row->CDARVPROD ?? null,
                "P_CDMARCA" => $row->CDMARCA ?? null,
                "P_DTENTREGA" => $row->DTENTREGA ?? null,
                "P_CDFILIAL" => $row->CDFILIAL ?? null,
                "P_TIPO" => $row->TIPO ?? null
            ];
            
            $std = $this->executeQuery($sql, $params);

           return $std->fetchAll();
        }
        catch(\Exception $e){
           throw $e;
        }
    }
    
}
