/**************************************************************************************************/
/*                                   Sistemas Integrados TECNISA                                  */
/*                                                                                                */
/*                                   PARMETROS DE PLANEJAMENTO                                   */
/*                                                                                                */
/**************************************************************************************************/
/*                                                                                                */
/* NOME: ATUALIZA_PARAMPLANEJA_PDR                                                                */
/*                                                                                                */
/* DESCRIO: ATUALIZA OS DADOS DA TABELA "PARAMPLANEJA" A PARTIR DO PARMETROS DA FILIAL PADRAO. */
/*                                                                                                */
/* CRIAO: 17/05/2009 - GLEYDSON ISRAEL DA SILVA GAZOLA                                          */
/*                                                                                                */
/**************************************************************************************************/
/* PARAMETROS: P_CDFILIAL - Cdigo da Filial                                                      */
/**************************************************************************************************/

IF EXISTS(SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'ATUALIZA_PARAMPLANEJA_PDR'  
             AND TYPE = 'P'
             AND U.UID = O.UID 
             AND U.NAME = USER)
   DROP PROCEDURE ATUALIZA_PARAMPLANEJA_PDR
GO

CREATE PROCEDURE ATUALIZA_PARAMPLANEJA_PDR(@P_CDFILIAL VARCHAR(4))
AS
   DECLARE @V_UPDATE_TEXT_PARAMPLANEJA     NVARCHAR(4000);
   DECLARE @V_INSERT_TEXT_PARAMPLANEJA     NVARCHAR(4000);
   DECLARE @V_CDFILIAL_PADRAO              VARCHAR(4);
   DECLARE @V_IDUTREQUPRAT_PADRAO          VARCHAR(1);
   DECLARE @V_CDFILCONSFIX_PADRAO          VARCHAR(4);
   DECLARE @V_CDPRACONSFIX_PADRAO          VARCHAR(10);
   DECLARE @V_CDSETCONPERC_PADRAO          VARCHAR(2);
   DECLARE @V_CDTCACONSFIX_PADRAO          VARCHAR(10);
   DECLARE @V_CDSERVRETPME_PADRAO          VARCHAR(5);
   DECLARE @V_CDSETCONFIXO_PADRAO          VARCHAR(2);
   DECLARE @V_CDFILCONSPER_PADRAO          VARCHAR(4);
   DECLARE @V_CDPRACONSPER_PADRAO          VARCHAR(10);
   DECLARE @V_CDTCACONSPER_PADRAO          VARCHAR(10);
   DECLARE @V_CDSETORCOZ_DESTINO           VARCHAR(2);
   DECLARE @V_CDTIPOCARD_DESTINO           VARCHAR(10);
   DECLARE @V_CDSERVFILI_DESTINO           VARCHAR(5);
   DECLARE @V_CDPRATO_DESTINO              VARCHAR(10);
   DECLARE @C_INSPARAM_CDFILIAL            VARCHAR(4);
   DECLARE @C_CAMPO_NAME                   SYSNAME;
BEGIN
   BEGIN TRANSACTION
      SELECT @V_CDFILIAL_PADRAO = CDFILIPADRAO
        FROM PARAMGERAL; 
      IF @V_CDFILIAL_PADRAO IS NOT NULL BEGIN 
         /* SELECIONA OS VALORES DA PARAMPLANEJA PARA A FILIAL PADRAO*/
         SELECT @V_IDUTREQUPRAT_PADRAO = IDUTREQUPRAT, 
                @V_CDFILCONSFIX_PADRAO = CDFILCONSFIX, 
                @V_CDPRACONSFIX_PADRAO = CDPRACONSFIX, 
                @V_CDSETCONPERC_PADRAO = CDSETCONPERC, 
                @V_CDTCACONSFIX_PADRAO = CDTCACONSFIX, 
                @V_CDSERVRETPME_PADRAO = CDSERVRETPME, 
                @V_CDSETCONFIXO_PADRAO = CDSETCONFIXO, 
                @V_CDFILCONSPER_PADRAO = CDFILCONSPER, 
                @V_CDPRACONSPER_PADRAO = CDPRACONSPER, 
                @V_CDTCACONSPER_PADRAO = CDTCACONSPER 
           FROM PARAMPLANEJA
          WHERE CDFILIAL = @V_CDFILIAL_PADRAO;
         /*SELECIONA AS FILIAIS QUE SERO ALTERADAS PARA UTLIZAREM OS PARAMETROS DA FILIAL PADRAO*/
         DECLARE C_INSPARAM CURSOR FOR (SELECT CDFILIAL
                                          FROM PARAMFILIPADR
                                         WHERE IDTABPARAMFILI = 'S'
                                           AND NMTABPARAMFILI = 'PARAMPLANEJA'
                                           AND CDFILIAL <> @V_CDFILIAL_PADRAO
                                           AND CDFILIAL = ISNULL(@P_CDFILIAL, CDFILIAL))
            OPEN C_INSPARAM;
            FETCH NEXT FROM C_INSPARAM INTO @C_INSPARAM_CDFILIAL;
            WHILE (@@FETCH_STATUS = 0) BEGIN
               SET @V_UPDATE_TEXT_PARAMPLANEJA = 'INSERT INTO PARAMPLANEJA (';
               SET @V_INSERT_TEXT_PARAMPLANEJA = '(SELECT ' ;
               /*SELECIONA OS CAMPOS DA TABELA 'PARAMPLANEJA'*/
               DECLARE C_CAMPO CURSOR FOR (SELECT NAME
                                             FROM SYSCOLUMNS
                                            WHERE ID = OBJECT_ID('PARAMPLANEJA'))
                  OPEN C_CAMPO;
                  FETCH NEXT FROM C_CAMPO INTO @C_CAMPO_NAME;
                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     /* SER FEITA A CONSISTNCIA DO BANCO, EVITADOR ASSIM GRAVAR DADOS INEXISTENTES PARA A FILIAL FILHA */
                     /* FKPLA00141SETORCOZ */
                     IF @C_CAMPO_NAME = 'CDSETCONPERC' BEGIN
                        SELECT @V_CDSETORCOZ_DESTINO = CDSETORCOZ
                          FROM SETORCOZ
                         WHERE CDFILIAL = @C_INSPARAM_CDFILIAL
                           AND CDSETORCOZ = @V_CDSETCONPERC_PADRAO;
                        IF @V_CDSETORCOZ_DESTINO IS NOT NULL BEGIN
                           SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                           SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                        END;
                     END ELSE BEGIN
                        /* FKPME00055TIPOCARD */
                        IF @C_CAMPO_NAME = 'CDTCACONSFIX' BEGIN
                           SELECT @V_CDTIPOCARD_DESTINO = CDTIPOCARD
                             FROM TIPOCARD
                            WHERE CDFILCENTPME = @C_INSPARAM_CDFILIAL
                              AND CDTIPOCARD = @V_CDTCACONSFIX_PADRAO;
                           IF @V_CDTIPOCARD_DESTINO IS NOT NULL BEGIN
                              SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                              SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                           END;
                        END ELSE BEGIN
                           /* FKPME00032SERVICO */
                           IF @C_CAMPO_NAME = 'CDSERVRETPME' BEGIN 
                              SELECT @V_CDSERVFILI_DESTINO = CDSERVFILI
                                FROM SERVICO
                               WHERE CDFILIAL = @C_INSPARAM_CDFILIAL
                                 AND CDSERVFILI = @V_CDSERVRETPME_PADRAO;
                              IF @V_CDSERVFILI_DESTINO IS NOT NULL BEGIN
                                 SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                 SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                              END;
                           END ELSE BEGIN
                              /* FKPLA00140SETORCOZ */
                              IF @C_CAMPO_NAME = 'CDSETCONFIXO' BEGIN
                                 SELECT @V_CDSETORCOZ_DESTINO = CDSETORCOZ
                                   FROM SETORCOZ
                                  WHERE CDFILIAL = @C_INSPARAM_CDFILIAL
                                    AND CDSETORCOZ = @V_CDSETCONFIXO_PADRAO;
                                 IF @V_CDSETORCOZ_DESTINO IS NOT NULL BEGIN
                                    SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                    SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                 END;
                              END ELSE BEGIN
                                 /* FKPLA00240PRATO */
                                 IF (@C_CAMPO_NAME = 'CDFILCONSPER') OR (@C_CAMPO_NAME = 'CDPRACONSPER') BEGIN
                                    IF @V_CDFILCONSPER_PADRAO = 'P' BEGIN
                                       SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                       SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                    END ELSE BEGIN
                                       SELECT @V_CDPRATO_DESTINO = CDPRATO
                                         FROM PRATO
                                        WHERE CDFILAUXPRAT = @C_INSPARAM_CDFILIAL
                                          AND CDPRATO = @V_CDPRACONSPER_PADRAO;
                                       IF @V_CDPRATO_DESTINO IS NOT NULL BEGIN
                                          SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                          SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                       END;
                                    END;
                                 END ELSE BEGIN
                                    /* FKPME00056TIPOCARD */
                                    IF @C_CAMPO_NAME = 'CDTCACONSPER' BEGIN
                                       SELECT @V_CDTIPOCARD_DESTINO = CDTIPOCARD
                                         FROM TIPOCARD
                                        WHERE CDFILCENTPME = @C_INSPARAM_CDFILIAL
                                          AND CDTIPOCARD = @V_CDTCACONSPER_PADRAO;
                                       IF @V_CDTIPOCARD_DESTINO IS NOT NULL BEGIN
                                          SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                          SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                       END;
                                    END ELSE BEGIN
                                       /* FKPLA00238PRATO */
                                       IF (@C_CAMPO_NAME = 'CDFILCONSFIX') OR (@C_CAMPO_NAME = 'CDPRACONSFIX') BEGIN
                                          IF @V_CDFILCONSFIX_PADRAO = 'P' BEGIN
                                             SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                             SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                          END ELSE BEGIN
                                             SELECT @V_CDPRATO_DESTINO = CDPRATO
                                               FROM PRATO
                                              WHERE CDFILAUXPRAT = @C_INSPARAM_CDFILIAL
                                                AND CDPRATO = @V_CDPRACONSFIX_PADRAO;
                                             IF @V_CDPRATO_DESTINO IS NOT NULL BEGIN
                                                SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                                SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                             END;
                                          END;
                                       END ELSE BEGIN
                                          IF NOT (@C_CAMPO_NAME = 'CDFILIAL') BEGIN
                                             SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                             SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + @C_CAMPO_NAME + ', ';
                                          END;   
                                       END;
                                    END;
                                 END;
                              END;
                           END;
                        END;
                     END;
                     FETCH NEXT FROM C_CAMPO INTO @C_CAMPO_NAME;
                  END;
               CLOSE C_CAMPO;
               DEALLOCATE C_CAMPO;
               /* DELETA O REGISTRO DA TABELA */
                DELETE FROM PARAMPLANEJA
                 WHERE CDFILIAL = @C_INSPARAM_CDFILIAL
               SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + 'CDFILIAL) ';
               SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + '''' + @C_INSPARAM_CDFILIAL + '''' + ' ';
               SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + 'FROM PARAMPLANEJA PA ';
               SET @V_INSERT_TEXT_PARAMPLANEJA = @V_INSERT_TEXT_PARAMPLANEJA + 'WHERE PA.CDFILIAL = ' + '''' + @V_CDFILIAL_PADRAO + '''' + ')';
               SET @V_UPDATE_TEXT_PARAMPLANEJA = @V_UPDATE_TEXT_PARAMPLANEJA + @V_INSERT_TEXT_PARAMPLANEJA;
               EXECUTE SP_EXECUTESQL @V_UPDATE_TEXT_PARAMPLANEJA;
               /* GRAVA 'S' PARA O CAMPO "IDUTREQUPRAT" PARA TODOS OS SERVIOS DA FILIAL CASO A FILIAL PADRAO UTILIZA
               A OPO DE "OBRIGAR A FAZER REQUISIO/RETIRADA POR PRATO" */
               IF @V_IDUTREQUPRAT_PADRAO = 'S' BEGIN
                  UPDATE SERVICO
                     SET IDUTRETIPRAT = 'S'
                   WHERE CDFILIAL = @C_INSPARAM_CDFILIAL;
               END;
               FETCH NEXT FROM C_INSPARAM INTO @C_INSPARAM_CDFILIAL;
            END;
         CLOSE C_INSPARAM;
         DEALLOCATE C_INSPARAM;
      END
   COMMIT TRANSACTION
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'ATUALIZA_PARAMPLANEJA_PDR','5.07.006';
GO
